﻿using UnityEngine;
using System.Collections;
using System.Collections.Generic;
using System;

public class MainCharControl : MonoBehaviour {

	private KeyBindings keyBindings = new KeyBindings();
	private List<string> actionsPendingToAssing = new List<string>();
	private Animator animator;

	private bool isWalking = false;
	private Vector3 faceScale;

	private Vector3 spawnPosition;
	private Vector3 backgroundPosition;

	private int lastFallTime = 0;
	private bool lastFall = false;


	// Use this for initialization
	void Start () {
		this.animator = gameObject.GetComponent<Animator> ();
		this.faceScale = transform.localScale;
		this.spawnPosition = transform.position;
		this.backgroundPosition = GameObject.Find("BackgroundChapu").transform.localScale;
	}

	public void SetAction(string action) {
		if (!keyBindings.ActionExists(action)) {
			throw new Exception("Action '"+action+"' does not exists");
		}
		actionsPendingToAssing.Add(action);
	}
	
	// Update is called once per frame
	void Update () {

		if (lastFall) {
			int fallingTime = Time.GetTimeStamp() - lastFallTime;
			if (fallingTime > 4) {
				lastFall = false;
				lastFallTime = Time.GetTimeStamp();
				Die();
			}
		}
		animator.SetBool("isFalling", IsFalling());
		animator.SetBool("isWalking", isWalking);
		isWalking = false;
		
		List<KeyCode> KeyPressed = KeyFetcher.GetPressedKeys();

		if (actionsPendingToAssing.Count > 0 && KeyPressed.Count > 0) {
			for (int i = 0; i < KeyPressed.Count; i++) {
				AssignAction( KeyPressed[i] );
			}
		} else {
			for (int i = 0; i < KeyPressed.Count; i++) {
				DoAction( keyBindings.GetActionFromKeysPressed(KeyPressed[i]) );
			}
		}
		
	}

	bool IsFalling() {
		bool isFalling = rigidbody2D.velocity.y != 0;

		if (isFalling && !lastFall) {
			lastFallTime = Time.GetTimeStamp();
		}

		if (!isFalling && lastFall) {
			lastFall = false;
		} else if (isFalling) {
			lastFall = true;
		}
		return isFalling;
	}

	void AssignAction(KeyCode KeyPressed) {
		if (keyBindings.SetKeyForAction(KeyPressed, actionsPendingToAssing[0])) {
			Debug.Log("Action = "+actionsPendingToAssing[0]+" Key = "+KeyPressed);
			actionsPendingToAssing.RemoveAt(0);
		}
	}

	void DoAction(string action) {

		Walk();

		if (action == KeyBindings.JUMP) {
			Jump();
			return;
		}

		if (action != null) {
			//Debug.Log( action );
		}
		
	}

	public void Die() {
		transform.position = this.spawnPosition;
	}

	void Walk() {

		

		List<KeyCode> KeyPressed = KeyFetcher.GetPressedKeys();
		List<string> actions = new List<string>();
		for (int i = 0; i < KeyPressed.Count; i++) {
			actions.Add( keyBindings.GetActionFromKeysPressed(KeyPressed[i]) );
		}
		float speed = 4f;

        if (Input.anyKey && KeyPressed.Count > 0) {
	        if (actions.Contains(KeyBindings.MOVE_LEFT))
	        {
        		transform.Translate(-Vector2.right * speed * UnityEngine.Time.deltaTime); 
        		Vector3 newScale = faceScale;
        		newScale.x *= -1;
    			transform.localScale = newScale;

    			Vector3 newBgScale = backgroundPosition;
    			newBgScale.x *= -1;

    			GameObject.Find("BackgroundChapu").transform.localScale = newBgScale; 
	        } 
	        else if (actions.Contains(KeyBindings.MOVE_RIGHT))
	        {
        		transform.Translate(Vector2.right * speed * UnityEngine.Time.deltaTime); 
    			transform.localScale = faceScale;
    			GameObject.Find("BackgroundChapu").transform.localScale = backgroundPosition; 
	        }

	        if (!IsFalling()) {
	        	isWalking = true;
	        }
	    }
	}

	void Jump() {
		if (IsFalling()) {
			return;
		}
		rigidbody2D.velocity = new Vector2(0, 0);
		rigidbody2D.AddForce(new Vector2(0, 10), ForceMode2D.Impulse);
	}
}
